from AutoRPE.UtilsRPE.PointerDealer import is_pointer
from AutoRPE.UtilsRPE.Functions import replace_comma_outside_parenthesis_by_hashtag, remove_literals
import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision
import re


def split_attributes(attributes):
    # Description: Given a string with declaration attributes separates the different attributes and
    # gives back a list of attributes
    # Input:
    #       -attributes: string with attributes
    # Output:
    #       -attributes_list: list containg strings with the different attributes
    attributes_list = replace_comma_outside_parenthesis_by_hashtag(attributes)
    attributes_list = [l.strip().replace("#", ",") for l in attributes_list.split(",")]
    return attributes_list


def which_type(attributes, variable=None):
    # Description: Given a list of attributes, this function returns the type of the variables
    # Input: List of attributes
    # Output: String with type
    if isinstance(attributes, str):
        attributes = split_attributes(attributes)
    assert isinstance(attributes, list)
    possible_types = ["real", "integer", "type", "class", "char", "logical", "complex", "double precision",
                      "procedure",
                      "useintrinsic",
                      "public"]
    # Type specification is always the first attribute
    t0 = attributes[0]

    var_type = None
    for ty in possible_types:
        if t0.lower().find(ty) == 0:
            var_type = ty
            break
    if not var_type:
        return None

    if var_type in ["type", "class"]:
        # It may be the case in which it is a type declaration instead of a variable declaration of a type
        # This workaround is to discard this cases.
        if not t0.count("("):
            return None
        var_type = t0[t0.find("(") + 1:-1].strip()
    elif var_type == "public":
        return None
    elif var_type == "real":
        pattern = r"real *\(([^\)]*)\)"
        if re.search(pattern, t0, re.I):
            type_specifier = re.search(pattern, t0, re.I).group(1).strip()
            try:
                return VariablePrecision.lookup_table[type_specifier]
            except KeyError:
                print("Key %s not found in lookup table.", type_specifier)
        else:
            if variable is not None:
                print("Type specifier not found in real variable " + variable + " declaration")
                return VariablePrecision.lookup_table['sp']
            else:
                raise AssertionError("Type specifier not found in real variable declaration")
    else:
        pass
    return var_type.lower()


def which_dimension(attributes, variable):
    # Description: Given a list of attributes, this function returns the dimension of the variables
    # Input: List of attributes
    # Output: Integer with the number of dimensions
    if isinstance(attributes, str):
        attributes = split_attributes(attributes)
    elif isinstance(attributes, list):
        pass
    else:
        raise AssertionError("Not a string nor a list, what it is?")
    dim = 0
    for att in attributes:
        if re.search(r"dimension\s*\(\*\)", att, re.I):
            dim = None
            break
        if att.lower().count("dimension"):
            dim = 1 + att.count(",")
            break
    if dim == 0:
        dim = variable.count(":")
        # TODO: Probably not the most robust solution to find the dimension
    if variable.count("="):
        variable = variable.split("=")[0].strip()
    if dim == 0 and variable.count("("):
        dim = variable.count(",") + 1

    return dim


def which_intent(attributes):
    # Description: Given a list of attributes, this function returns the intent of the variables
    # Input: List of attributes
    # Output: String with the intent of the variable, if intent is not specified the default is inout
    if isinstance(attributes, str):
        attributes = split_attributes(attributes)
    elif isinstance(attributes, list):
        pass
    intent = "inout"
    for att in attributes:
        if att.lower().count("intent"):
            intent = att
            break
    if intent.count("("):
        intent = intent[intent.find("(") + 1:-1].strip().lower()
    return intent


def is_optional(attributes):
    # Description: Given a list of attributes, this function returns a boolean describing if the variable is optional
    # Input: List of attributes
    # Output: Boolean about optionality of variable
    attributes = split_attributes(attributes)
    optional = False
    for att in attributes:
        if att.lower().count("optional"):
            optional = True
            break
    return optional


def is_parameter(attributes):
    # Description: Given a list of attributes, this function returns a boolean describing if the variable is optional
    # Input: List of attributes
    # Output: Boolean about optionality of variable
    attributes = split_attributes(attributes)
    for att in attributes:
        if att.lower().count("parameter"):
            return True
    return False


def is_allocatable(attributes):
    # Description: Given a list of attributes, this function returns a boolean describing if the variable is allocatable
    # Input: List of attributes
    # Output: Boolean about if variable is allocatable
    attributes = split_attributes(attributes)
    for att in attributes:
        if att.lower().count("allocatable"):
            return True
    return False


def parse_attributes(attributes, variables):
    if isinstance(attributes, str):
        attributes = split_attributes(attributes)
    assert isinstance(attributes, list)
    var_type = which_type(attributes, variables)
    dimension = which_dimension(attributes, variables)
    intent = which_intent(attributes)
    optional = is_optional(attributes)
    allocatable = is_allocatable(attributes)
    pointer = is_pointer(attributes, variables)
    parameter = is_parameter(attributes)
    return var_type, dimension, intent, optional, allocatable, pointer, parameter


def split_declaration_arguments(attributes):
    # Description: given a string containing arguments, return a list with arguments
    # Input: String with arguments
    # Output: List with arguments
    # TODO: If the string of arguments contains parenthesis, we should keep information about the dimensions.
    # TODO: If the argument has a value declaration, we should get rid of this.

    attributes = remove_literals(attributes)
    attributes = replace_comma_outside_parenthesis_by_hashtag(attributes)
    attributes = [l.strip().replace("#", ",") for l in attributes.split(",")]
    attributes = [el[:el.find("(")] if el.count("(") else el for el in attributes]
    attributes = [el[:el.find("=")].strip() if el.count("=") else el for el in attributes]
    attributes = [item.replace(" ", "") for item in attributes]
    return attributes
